package org.coderu.common.utils.types;

import java.io.Serializable;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;


/**
 *
 * a pair of two objects
 *
 * @author Alexander Rubinov
 *
 * @param <T1> type of first object
 * @param <T2> type of second object
 */
@SuppressWarnings("serial")
public abstract class AbstractPair<T1,T2> implements Serializable{

    private final T1 first;
    private final T2 second;

    protected AbstractPair(T1 first, T2 second) {
        this.first=first;
        this.second=second;
    }

    /**
     * get first object
     * @return first object
     */
    public T1 getFirst() {
        return first;
    }

    /**
     * get second object
     * @return second object
     */
    public T2 getSecond() {
        return second;
    }

    /**
     * {@inheritDoc}
     */
    public String toString(){
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
        .append(first)
        .append(second)
        .toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        return new HashCodeBuilder(31,37)
        .append(first)
        .append(second)
        .toHashCode();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) { return false; }
        if (obj == this) { return true; }
        if (obj.getClass() != getClass()) {return false;}
        
        @SuppressWarnings("rawtypes")
        final AbstractPair rhs = (AbstractPair) obj;
        return new EqualsBuilder()
        .append(first, rhs.first)
        .append(second, rhs.second)
        .isEquals();

    }

}
