package org.coderu.common.utils.types;

/**
 *
 * a triple of objects
 *
 * @author Alexander Rubinov
 *
 * @param <T1> type of first object
 * @param <T2> type of second object
 * @param <T3> type of third object
 */
public abstract class AbstractTriple<T1,T2,T3> {

    private final T1 first;
    private final T2 second;
    private final T3 third;

    protected AbstractTriple(T1 first, T2 second, T3 third) {
        this.first=first;
        this.second=second;
        this.third=third;
    }

    public T1 getFirst() {
        return first;
    }

    public T2 getSecond() {
        return second;
    }

    public T3 getThird() {
        return third;
    }

    public String toString(){
        return String.format("[%s,%s,%s]", first, second, third);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        final int prime1 = 11;
        final int prime2 = 17;
        final int prime3 = 31;
        return  prime1 * (prime1 + (first!=null?first.hashCode():0) ) +
                prime2 * (prime2 + (second!=null?second.hashCode():0)) +
                prime3 * (prime3 + (third!=null?third.hashCode():0));
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if(this.getClass() != obj.getClass() ){
            return false;
        }
        @SuppressWarnings("rawtypes")
        final AbstractTriple other = (AbstractTriple)obj;
        return  this == other || (
                nullableEqual(first,other.first) &&
                nullableEqual(second,other.second ) &&
                nullableEqual(third,other.third));
    }

    private static boolean nullableEqual(Object o1, Object o2){
        return (o1 != null && o2 != null && o1.equals(o2)) || (o1==null && o2==null);
    }
    
}