package org.coderu.core.api;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Nullable;

import org.apache.commons.lang3.Validate;
import org.coderu.common.utils.types.SingleValue;

import com.google.common.base.Function;
import com.google.common.collect.Ordering;

@SuppressWarnings("serial")
public final class Packagge extends SingleValue<String> implements
        Comparable<Packagge> {

    public static final Ordering<Packagge> ORDERING = new Ordering<Packagge>() {

        @Override
        public int compare(@Nullable Packagge left, @Nullable Packagge right) {
            return left.compareTo(right);
        }
    };

    public static final Function<String, Packagge> FROM_STRING = new Function<String, Packagge>() {

        @Override
        public Packagge apply(@Nullable String arg0) {
            return new Packagge(arg0);
        }
    };

    public Packagge(final String name) { // NO_UCD
        super(name);
        checkValidPackageName(name);
    }

    public String getName() {
        return getValue();
    }

    public int compareTo(Packagge o) {
        return getValue().compareTo(o.getValue());
    }

    private static void checkValidPackageName(final String packageName) {
        final Matcher matcher = PACKAGE_NAME.matcher(packageName);
        Validate.isTrue(matcher.matches(),
                String.format("package name \"%s\" is not valid", packageName));
    }

    private static final Pattern PACKAGE_NAME = Pattern
            .compile("^$|^[a-z_A-Z][a-z0-9_A-Z]*(\\.[a-z_A-Z][a-z0-9_A-Z]*)*$");
}
