package org.coderu.core.impl.explorer.helpers;

import static org.coderu.common.utils.CollectionUtils.*;

import java.util.Collection;

import org.coderu.common.utils.CollectionUtils;
import org.coderu.common.utils.MapUtils;
import org.coderu.common.utils.PairUtils;
import org.coderu.common.utils.types.Pair;
import org.coderu.core.api.Clazz;
import org.coderu.core.api.Packagge;
import org.coderu.core.impl.common.types.ClassDependencies;
import org.coderu.core.impl.common.types.ClassDependency;
import org.coderu.core.impl.common.types.ClassName;
import org.coderu.core.impl.common.types.PackageDependency;
import org.coderu.core.impl.common.types.PackageName;
import org.coderu.core.impl.common.types.PackageWithDependencies;

import com.google.common.base.Function;
import com.google.common.base.Functions;
import com.google.common.base.Predicate;
import static com.google.common.base.Predicates.*;
import com.google.common.collect.Collections2;


public final class PackageToClassDependency{

    public static Function<PackageDependency,Collection<ClassDependency>> create(
        final Collection<PackageWithDependencies> jpackages){
        
        final Function<Packagge,PackageWithDependencies> jpackageByName =
            Functions.forMap(MapUtils.mapBy(jpackages, PackageWithDependencies.TO_PACKAGE));
        
        return new Function<PackageDependency,Collection<ClassDependency>>(){
            public Collection<ClassDependency> apply(PackageDependency o) {
            
                final PackageWithDependencies from = 
                    jpackageByName.apply(new Packagge(o.getFirst().getFullName()));

                final Collection<ClassDependencies> fromClassDependencies = from.getSecond();
                
                final Collection<Pair<Clazz,Clazz>> expanded = 
                    CollectionUtils.concat(
                        transform(fromClassDependencies, ClassDependencies.EXPAND));
                
                final Collection<Pair<Clazz, Clazz>> selected = 
                    Collections2.filter(expanded, compose(belongToPackage(o.getTo()), PAIR2SECOND));
                
                return transform(selected, PAIR_TO_CLASS_DEPENDENCY);
            }
        };
    }
    
    private static final Function<Pair<Clazz, Clazz>,ClassDependency> PAIR_TO_CLASS_DEPENDENCY = new
        Function<Pair<Clazz,Clazz>, ClassDependency>() {
            
            public ClassDependency apply(Pair<Clazz, Clazz> o) {
                return new ClassDependency(
                    toClassName(o.getFirst()),toClassName(o.getSecond()));
            }
        };
        
    private static final  Function<Pair<Clazz, Clazz>, Clazz> PAIR2SECOND = 
        PairUtils.<Clazz,Clazz>pair2second();

    
    
    private static final Predicate<Clazz> belongToPackage(final PackageName packagge){
        return new Predicate<Clazz>() {
            
            public boolean apply(Clazz o) {
            
                return className2Package(o.getName()).equals(packagge.getFullName());
            }
        };
    }

    
    private static String className2Package(final String className){
        int index = className.lastIndexOf(".");
        return className.substring(0, index);
    }


    private static ClassName toClassName(Clazz o) {

        return new ClassName(o.getName());
    }
    
}
